use toml::Table;

use crate::config_file_parsing::*;

#[derive(Debug, Clone)]
pub struct ConfigTable<'a> {
    pub build_dir: &'a str,
    pub compile_commands: bool,
    pub compile_commands_target: &'a str,
}

impl<'a> ConfigTable<'a> {
    pub fn get_default() -> Self {
        ConfigTable {
            build_dir: "build",
            compile_commands: false,
            compile_commands_target: "main",
        }
    }

    pub fn from_table(table: &'a Table, key_path: String) -> Result<Self, ConfigFileError<'a>> {
        let mut ret = Self::get_default();

        if let Some(v) = get_key_as_str(table, "build_dir", format!("{key_path}.build_dir"))? {
            ret.build_dir = v;
        }

        if let Some(v) = get_key_as_bool(
            table,
            "compile_commands",
            format!("{key_path}.compile_commands"),
        )? {
            ret.compile_commands = v;
        }

        if let Some(v) = get_key_as_str(
            table,
            "compile_commands_target",
            format!("{key_path}.compile_commands_target"),
        )? {
            ret.compile_commands_target = v;
        }
        Ok(ret)
    }
}
