use toml::Table;

use crate::config_file_parsing::*;
use crate::target_options::*;

#[derive(Debug, Clone)]
pub struct Target<'a> {
    pub name: &'a str,
    pub outfile: &'a str,
    pub compiler: &'a str,
    pub compiler_flags: Vec<&'a str>,
    pub linker: &'a str,
    pub linker_flags: Vec<&'a str>,
    pub linker_libs: Vec<&'a str>,
    pub sources: Vec<&'a str>,
    pub is_default: bool,
    pub opts: TargetOptions<'a>,
}

impl<'a> Target<'a> {
    pub fn get_default() -> Self {
        Target {
            name: "",
            outfile: "a.out",
            compiler: "cc",
            compiler_flags: vec![""],
            linker: "cc",
            linker_flags: vec![""],
            linker_libs: vec![""],
            sources: vec![""],
            is_default: false,
            opts: TargetOptions::get_default(),
        }
    }
    pub fn from_table(
        table: &'a Table,
        name: &'a str,
        key_path: String,
    ) -> Result<Self, ConfigFileError<'a>> {
        let mut ret = Self::get_default();

        if let Some(v) = get_key_as_table(table, "opts", format!("{key_path}.opts"))? {
            ret.opts = TargetOptions::from_table(&v, format!("{key_path}.opts"))?;
        }

        ret.name = name;

        if let Some(v) = get_key_as_str(table, "outfile", format!("{key_path}.outfile"))? {
            ret.outfile = v;
        }

        if let Some(v) = get_key_as_str(table, "compiler", format!("{key_path}.compiler"))? {
            ret.compiler = v;
        }

        if let Some(v) = get_key_as_vec_str(
            table,
            "compiler_flags",
            format!("{key_path}.compiler_flags"),
        )? {
            ret.compiler_flags = v;
        }

        if let Some(v) = get_key_as_str(table, "linker", format!("{key_path}.linker"))? {
            ret.linker = v;
        }

        if let Some(v) =
            get_key_as_vec_str(table, "linker_flags", format!("{key_path}.linker_flags"))?
        {
            ret.linker_flags = v;
        }

        if let Some(v) =
            get_key_as_vec_str(table, "linker_libs", format!("{key_path}.linker_libs"))?
        {
            ret.linker_libs = v;
        }

        if let Some(v) = get_key_as_vec_str(table, "sources", format!("{key_path}.sources"))? {
            ret.sources = v;
        }

        if let Some(v) = get_key_as_bool(table, "is_default", format!("{key_path}.is_default"))? {
            ret.is_default = v;
        }

        Ok(ret)
    }
}
