use toml::{Table, Value};

use crate::config_file_parsing::*;

#[derive(Debug, Clone)]
pub struct TargetOptions<'a> {
    pub inherit: bool,
    pub inherit_from: &'a str,
}

impl<'a> TargetOptions<'a> {
    pub fn get_default() -> Self {
        TargetOptions {
            inherit: true,
            inherit_from: "main",
        }
    }
    pub fn from_table(table: &'a Table, key_path: String) -> Result<Self, ConfigFileError<'a>> {
        let mut ret = Self::get_default();

        if let Some(v) = get_key_as_bool(table, "inherit", format!("{key_path}.inherit"))? {
            ret.inherit = v;
        }
        if let Some(v) = get_key_as_str(table, "inherit_from", format!("{key_path}.inherit_from"))?
        {
            ret.inherit_from = v;
        }
        Ok(ret)
    }
}
